#! /bin/sh
# $Id: mail.sh,v 1.4 2001/09/09 09:35:27 gerhard Exp $
# Copyright Gerhard Rieger 2001
# Published under the GNU General Public License V.2, see file COPYING

#set -vx

# This is an example for a shell script that can be fed to socat with exec.
# Its clue is that it does not use stdin/stdout for communication with socat,
# so you may feed the mail message via stdin to the script.
# It supports multiline answers from server.

# Usage:  cat message.txt |socat [-v] exec:"mail.sh target@domain.com",fdin=3,fdout=4 tcp:mail.relay.org:25,crlf

rcpt="$1"
[ -z "$1" ] && rcpt="root@loopback"
server=$(expr "$rcpt" : '[^@]*@\(.*\)')

# this function waits for a complete server message, checks if its status
# is in the permitted range (terminates session if not), and returns.
function mail_chat {
    local cmd="$1" 
    local errlevel="$2";  [ -z "$errlevel" ] && errlevel=300

    echo "> $cmd" >&2
    if [ -n "$cmd" ]; then echo "$cmd" >&4; fi
    while read status message <&3;
	(
	    case "$status" in
	    [0-9][0-9][0-9]-*) exit 0;;
	    [0-9][0-9][0-9]*) exit 1;;
	    *) exit 0;;
	    esac
	)
    do :; done
    echo "< $status $message" >&2
    if [ "$status" -ge "$errlevel" ]; then
	echo $message >&2
	echo "QUIT" >&4; exit 1
    fi
}


# expect server greeting
mail_chat

mail_chat "HELO $(hostname)"

mail_chat "MAIL FROM: sender@$(hostname)"

mail_chat "RCPT to: $rcpt"

mail_chat "DATA" 400

while read l; do echo "$l" >&4; done
mail_chat "."

mail_chat "QUIT"

exit 0
